import shutil
import sys
from os import walk
import logging
from pathlib import Path
import functools
from typing import Union

import numpy as np
import matplotlib.pyplot as plt
from matplotlib.widgets import Button
from scipy import ndimage

from cgmml.common.depthmap_toolkit.depthmap import Depthmap
from cgmml.common.depthmap_toolkit.exporter import export_obj, export_pcd
from cgmml.common.depthmap_toolkit.visualisation import render_plot

logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)
handler = logging.StreamHandler()
handler.setFormatter(logging.Formatter('%(asctime)s - %(levelname)s - %(message)s - %(pathname)s: line %(lineno)d'))
logger.addHandler(handler)

REPO_DIR = Path(__file__).parents[3]
EXPORT_DIR = REPO_DIR / 'data' / 'export'

LAST_CLICK_COORD = [0, 0, 0]
IDX_CUR_DMAP = None
DMAP = None


def onclick(event):
    global DMAP
    global LAST_CLICK_COORD
    if event.xdata is not None and event.ydata is not None:
        x = int(event.ydata)
        y = int(event.xdata)
        if x > 1 and y > 1 and x < DMAP.width - 2 and y < DMAP.height - 2:
            depth = DMAP.depthmap_arr[x, y]
            if not depth:
                logger.info('no valid data')
                return
            res = DMAP.convert_2d_to_3d(1, x, y, depth)
            diff: list = [LAST_CLICK_COORD[0] - res[0], LAST_CLICK_COORD[1] - res[1], LAST_CLICK_COORD[2] - res[2]]
            dst: float = np.sqrt(diff[0] * diff[0] + diff[1] * diff[1] + diff[2] * diff[2])
            logger.info(f'x={res[0]:.3f}, y={res[1]:.3f}, depth={res[2]:.3f}, diff={dst:.3f}')
            LAST_CLICK_COORD[0] = res[0]
            LAST_CLICK_COORD[1] = res[1]
            LAST_CLICK_COORD[2] = res[2]


def export_object(event):
    global DMAP
    floor = DMAP.get_floor_level()
    export_obj(EXPORT_DIR / f'output{IDX_CUR_DMAP}.obj', DMAP, floor, triangulate=True)


def export_pointcloud(event):
    global DMAP
    export_pcd(EXPORT_DIR / f'output{IDX_CUR_DMAP}.pcd', DMAP)


def next_click(event, calibration_fpath: str, depthmap_dir: str):
    global IDX_CUR_DMAP
    IDX_CUR_DMAP = IDX_CUR_DMAP + 1
    if (IDX_CUR_DMAP == size):
        IDX_CUR_DMAP = 0
    show(depthmap_dir, calibration_fpath)


def prev_click(event, calibration_fpath: str, depthmap_dir: str):
    global IDX_CUR_DMAP
    IDX_CUR_DMAP = IDX_CUR_DMAP - 1
    if (IDX_CUR_DMAP == -1):
        IDX_CUR_DMAP = size - 1
    show(depthmap_dir, calibration_fpath)


def show(depthmap_dir: str, calibration_file: str, original_orientation=False):
    global DMAP
    fig.canvas.manager.set_window_title(depth_filenames[IDX_CUR_DMAP])
    rgb_filename = rgb_filenames[IDX_CUR_DMAP] if rgb_filenames else 0
    DMAP = Depthmap.create_from_zip(depthmap_dir,
                                    depth_filenames[IDX_CUR_DMAP],
                                    rgb_filename,
                                    calibration_file)

    angle = DMAP.get_angle_between_camera_and_floor()
    logging.info('angle between camera and floor is %f', angle)

    output = render_plot(DMAP)
    if original_orientation:
        output = ndimage.rotate(output, 90)
    plt.imshow(output)
    plot_names = ['depth', 'normals', 'child/background segmentation', 'confidence']
    if DMAP.has_rgb:
        plot_names.append('rgb')
    plot_title = '\n'.join([f'{i}: {plot_name}' for i, plot_name in enumerate(plot_names)])
    plt.title(plot_title)
    plt.show()


def is_legit_file(fpath: Union[str, Path]) -> bool:
    """Find non-hidden files"""
    if Path(fpath).name.startswith('.'):
        return False
    return True


def assemble_filenames(input_dir: Path):
    """Inspect input dir for files and return a sorted list of those files"""
    all_filenames = []
    for (_dirpath, _dirnames, filenames) in walk(input_dir):
        fns = list(filter(is_legit_file, filenames))
        all_filenames.extend(fns)
    all_filenames.sort()
    return all_filenames


if __name__ == "__main__":
    if len(sys.argv) != 3:
        print('You did not enter depthmap_dir folder and calibration file path')
        print('E.g.: python toolkit.py depthmap_dir calibration_fpath')
        sys.exit(1)

    depthmap_dir = sys.argv[1]
    calibration_fpath = sys.argv[2]

    depth_dir = Path(depthmap_dir) / 'depth'
    rgb_dir = Path(depthmap_dir) / 'rgb'
    assert depth_dir.exists(), depthmap_dir
    if not rgb_dir.exists():
        logging.warn("RGB directory doesn't exist. Working with depth data only")

    depth_filenames = assemble_filenames(depth_dir)
    rgb_filenames = assemble_filenames(rgb_dir)

    # Re-create export folder
    try:
        shutil.rmtree(EXPORT_DIR)
    except BaseException:
        print('no previous data to delete')
    EXPORT_DIR.mkdir(parents=True, exist_ok=True)

    # Show viewer
    IDX_CUR_DMAP = 0
    size = len(depth_filenames)
    fig = plt.figure()
    fig.canvas.mpl_connect('button_press_event', functools.partial(onclick))
    bprev = Button(plt.axes([0.0, 0.0, 0.1, 0.075]), '<<', color='gray')
    bprev.on_clicked(functools.partial(prev_click, calibration_fpath=calibration_fpath, depthmap_dir=depthmap_dir))
    bnext = Button(plt.axes([0.9, 0.0, 0.1, 0.075]), '>>', color='gray')
    bnext.on_clicked(functools.partial(next_click, calibration_fpath=calibration_fpath, depthmap_dir=depthmap_dir))
    bexport_obj = Button(plt.axes([0.3, 0.0, 0.2, 0.05]), 'Export OBJ', color='gray')
    bexport_obj.on_clicked(functools.partial(export_object))
    bexport_pcd = Button(plt.axes([0.5, 0.0, 0.2, 0.05]), 'Export PCD', color='gray')
    bexport_pcd.on_clicked(functools.partial(export_pointcloud))
    background = Button(plt.axes([0.0, 0.0, 1.0, 1.0]), '', color='white')
    show(depthmap_dir, calibration_fpath)



from typing import Tuple

import tensorflow as tf
from tensorflow.keras import models, layers


def create_cnn(input_shape: Tuple[int], dropout: bool) -> tf.Module:
    model = models.Sequential()

    model.add(layers.Conv2D(filters=32, kernel_size=(3, 3), padding="same", activation="relu", input_shape=input_shape))
    model.add(layers.Conv2D(filters=32, kernel_size=(3, 3), padding="same", activation="relu"))
    model.add(layers.MaxPooling2D(pool_size=(2, 2)))
    if dropout:
        model.add(layers.Dropout(0.05))

    model.add(layers.Conv2D(filters=64, kernel_size=(3, 3), padding="same", activation="relu"))
    model.add(layers.Conv2D(filters=64, kernel_size=(3, 3), padding="same", activation="relu"))
    model.add(layers.MaxPooling2D(pool_size=(2, 2)))
    if dropout:
        model.add(layers.Dropout(0.075))

    model.add(layers.Conv2D(filters=128, kernel_size=(3, 3), padding="same", activation="relu"))
    model.add(layers.Conv2D(filters=128, kernel_size=(3, 3), padding="same", activation="relu"))
    model.add(layers.MaxPooling2D(pool_size=(2, 2)))
    if dropout:
        model.add(layers.Dropout(0.1))

    model.add(layers.Conv2D(filters=128, kernel_size=(3, 3), padding="same", activation="relu"))
    model.add(layers.Conv2D(filters=128, kernel_size=(3, 3), padding="same", activation="relu"))
    model.add(layers.MaxPooling2D(pool_size=(2, 2)))
    if dropout:
        model.add(layers.Dropout(0.125))

    model.add(layers.Conv2D(filters=256, kernel_size=(3, 3), padding="same", activation="relu"))
    model.add(layers.Conv2D(filters=256, kernel_size=(3, 3), padding="same", activation="relu"))
    model.add(layers.MaxPooling2D(pool_size=(2, 2)))
    if dropout:
        model.add(layers.Dropout(0.15))

    model.add(layers.Conv2D(filters=256, kernel_size=(3, 3), padding="same", activation="relu"))
    model.add(layers.Conv2D(filters=256, kernel_size=(3, 3), padding="same", activation="relu"))
    model.add(layers.MaxPooling2D(pool_size=(2, 2)))
    if dropout:
        model.add(layers.Dropout(0.175))

    model.add(layers.Flatten())

    model.add(layers.Dense(1024, activation="relu"))
    if dropout:
        model.add(layers.Dropout(0.2))

    model.add(layers.Dense(128, activation="relu"))
    if dropout:
        model.add(layers.Dropout(0.25))

    model.add(layers.Dense(1, activation="linear"))

    return model


def create_base_cnn(input_shape: Tuple[int], dropout: bool) -> tf.Module:
    model = models.Sequential()

    model.add(layers.Conv2D(filters=32, kernel_size=(3, 3), padding="same", activation="relu", input_shape=input_shape))
    model.add(layers.Conv2D(filters=32, kernel_size=(3, 3), padding="same", activation="relu"))
    model.add(layers.MaxPooling2D(pool_size=(2, 2)))
    if dropout:
        model.add(layers.Dropout(0.05))

    model.add(layers.Conv2D(filters=64, kernel_size=(3, 3), padding="same", activation="relu"))
    model.add(layers.Conv2D(filters=64, kernel_size=(3, 3), padding="same", activation="relu"))
    model.add(layers.MaxPooling2D(pool_size=(2, 2)))
    if dropout:
        model.add(layers.Dropout(0.075))

    model.add(layers.Conv2D(filters=128, kernel_size=(3, 3), padding="same", activation="relu"))
    model.add(layers.Conv2D(filters=128, kernel_size=(3, 3), padding="same", activation="relu"))
    model.add(layers.MaxPooling2D(pool_size=(2, 2)))
    if dropout:
        model.add(layers.Dropout(0.1))

    model.add(layers.Conv2D(filters=128, kernel_size=(3, 3), padding="same", activation="relu"))
    model.add(layers.Conv2D(filters=128, kernel_size=(3, 3), padding="same", activation="relu"))
    model.add(layers.MaxPooling2D(pool_size=(2, 2)))
    if dropout:
        model.add(layers.Dropout(0.125))

    model.add(layers.Conv2D(filters=256, kernel_size=(3, 3), padding="same", activation="relu"))
    model.add(layers.Conv2D(filters=256, kernel_size=(3, 3), padding="same", activation="relu"))
    model.add(layers.MaxPooling2D(pool_size=(2, 2)))
    if dropout:
        model.add(layers.Dropout(0.15))

    model.add(layers.Conv2D(filters=256, kernel_size=(3, 3), padding="same", activation="relu"))
    model.add(layers.Conv2D(filters=256, kernel_size=(3, 3), padding="same", activation="relu"))
    model.add(layers.MaxPooling2D(pool_size=(2, 2)))
    if dropout:
        model.add(layers.Dropout(0.175))

    model.add(layers.Flatten())

    model.add(layers.Dense(1024, activation="relu"))
    if dropout:
        model.add(layers.Dropout(0.2))

    model.add(layers.Dense(128, activation="relu"))
    if dropout:
        model.add(layers.Dropout(0.25))

    return model


def create_head(input_shape: Tuple[int], dropout: bool, name: str = "head") -> tf.Module:
    model = models.Sequential(name=name)
    model.add(layers.Dense(128, activation="relu", input_shape=input_shape))
    if dropout:
        model.add(layers.Dropout(0.2))

    model.add(layers.Dense(64, activation="relu"))
    if dropout:
        model.add(layers.Dropout(0.2))

    model.add(layers.Dense(16, activation="relu"))
    if dropout:
        model.add(layers.Dropout(0.2))

    model.add(layers.Dense(1, activation="linear"))
    return model



import argparse
import logging
from typing import List

from glob2 import glob
import pandas as pd

OUTPUT_FILE_NAME = 'evaluated_models_result.csv'

logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)
handler = logging.StreamHandler()
handler.setFormatter(logging.Formatter('%(asctime)s - %(levelname)s - %(message)s - %(pathname)s: line %(lineno)d'))
logger.addHandler(handler)


def combine_model_results(csv_file_list: List[str], output_path: str):
    """Function to combine the models resultant csv files into a single file

    Args:
        csv_file_list: list containing absolute path of csv file
        output_path: target folder path where to save result csv file
    """
    if len(csv_file_list) <= 0:
        logger.warning("No csv files found in output directory to combine")
        return
    result_list = [pd.read_csv(results, index_col=0) for results in csv_file_list]
    final_result = pd.concat(result_list, axis=0)
    final_result = final_result.rename_axis("Model")
    final_result = final_result.round(2)
    result_csv = f"{output_path}/{OUTPUT_FILE_NAME}"
    final_result.to_csv(result_csv, index=True)


if __name__ == "__main__":
    paths = {
        'height': 'outputs/height',
        'weight': 'outputs/weight'
    }

    def validate_arg(args_string: str) -> str:
        """Validate the passed arguments

        Args:
            arg_string: input argument

        Raises:
            argparse.ArgumentTypeError: error to throw if the value is not valid

        Returns: args_string
        """
        if args_string not in paths.keys():
            raise argparse.ArgumentTypeError(
                f"{args_string} is an invalid argument value. Valid options ared: {paths.keys()}")
        return args_string

    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--model_measurement",
        default="height",
        type=validate_arg,
        help="defining models usage for the measuring height or weight")
    args = parser.parse_args()
    result_path = paths.get(args.model_measurement)
    csv_path = f"{result_path}/*.csv"
    csv_files = glob(csv_path)
    combine_model_results(csv_files, result_path)



import os
import tarfile
import time
from typing import Tuple, Any

from matplotlib import gridspec
import matplotlib.pyplot as plt
import numpy as np
from PIL import Image
import pickle

import tensorflow.compat.v1 as tf
tf.disable_v2_behavior()


class DeepLabModel(object):
    """Class to load deeplab model and run inference."""

    INPUT_TENSOR_NAME = 'ImageTensor:0'
    OUTPUT_TENSOR_NAME = 'SemanticPredictions:0'
    INPUT_SIZE = 513
    FROZEN_GRAPH_NAME = 'frozen_inference_graph'

    def __init__(self, tarball_path):
        """Creates and loads pretrained deeplab model."""
        self.graph = tf.Graph()

        graph_def = None
        # Extract frozen graph from tar archive.
        tar_file = tarfile.open(tarball_path)
        for tar_info in tar_file.getmembers():
            if self.FROZEN_GRAPH_NAME in os.path.basename(tar_info.name):
                file_handle = tar_file.extractfile(tar_info)
                graph_def = tf.GraphDef.FromString(file_handle.read())
                break

        tar_file.close()

        if graph_def is None:
            raise RuntimeError('Cannot find inference graph in tar archive.')

        with self.graph.as_default():
            tf.import_graph_def(graph_def, name='')

        self.sess = tf.Session(graph=self.graph)

    def run(self, image):
        """Runs inference on a single image.

        Args:
            image: A PIL.Image object, raw input image.

        Returns:
            resized_image: RGB image resized from original input image.
            seg_map: Segmentation map of `resized_image`.
        """
        width, height = image.size
        resize_ratio = 1.0 * self.INPUT_SIZE / max(width, height)
        target_size = (int(resize_ratio * width), int(resize_ratio * height))
        resized_image = image.convert('RGB').resize(target_size, Image.ANTIALIAS)
        batch_seg_map = self.sess.run(self.OUTPUT_TENSOR_NAME,
                                      feed_dict={self.INPUT_TENSOR_NAME: [np.asarray(resized_image)]})
        seg_map = batch_seg_map[0]
        return resized_image, seg_map


def create_pascal_label_colormap():
    """Creates a label colormap used in PASCAL VOC segmentation benchmark.

    Returns:
        A Colormap for visualizing segmentation results.
    """
    colormap = np.zeros((256, 3), dtype=int)
    ind = np.arange(256, dtype=int)

    for shift in reversed(range(8)):
        for channel in range(3):
            colormap[:, channel] |= ((ind >> channel) & 1) << shift
        ind >>= 3

    return colormap


def label_to_color_image(label):
    """Adds color defined by the dataset colormap to the label.

    Args:
        label: A 2D array with integer type, storing the segmentation label.

    Returns:
        result: A 2D array with floating type. The element of the array
            is the color indexed by the corresponding element in the input label
            to the PASCAL color map.

    Raises:
        ValueError: If label is not of rank 2 or its value is larger than color
            map maximum entry.
    """
    if label.ndim != 2:
        raise ValueError('Expect 2-D input label')

    colormap = create_pascal_label_colormap()

    if np.max(label) >= len(colormap):
        raise ValueError('label value too large.')

    return colormap[label]


def vis_segmentation(image, seg_map):
    """Visualizes input image, segmentation map and overlay view."""
    plt.figure(figsize=(15, 5))
    grid_spec = gridspec.GridSpec(1, 4, width_ratios=[6, 6, 6, 1])

    plt.subplot(grid_spec[0])
    plt.imshow(image)
    plt.axis('off')
    plt.title('input image')

    plt.subplot(grid_spec[1])
    seg_image = label_to_color_image(seg_map).astype(np.uint8)
    plt.imshow(seg_image)
    plt.axis('off')
    plt.title('segmentation map')

    plt.subplot(grid_spec[2])
    plt.imshow(image)
    plt.imshow(seg_image, alpha=0.7)
    plt.axis('off')
    plt.title('segmentation overlay')

    unique_labels = np.unique(seg_map)
    ax = plt.subplot(grid_spec[3])
    plt.imshow(FULL_COLOR_MAP[unique_labels].astype(np.uint8), interpolation='nearest')
    ax.yaxis.tick_right()
    plt.yticks(range(len(unique_labels)), LABEL_NAMES[unique_labels])
    plt.xticks([], [])
    ax.tick_params(width=0.0)
    plt.grid('off')
    plt.show()


LABEL_NAMES = np.asarray([
    'background', 'aeroplane', 'bicycle', 'bird', 'boat', 'bottle', 'bus',
    'car', 'cat', 'chair', 'cow', 'diningtable', 'dog', 'horse', 'motorbike',
    'person', 'pottedplant', 'sheep', 'sofa', 'train', 'tv'
])

FULL_LABEL_MAP = np.arange(len(LABEL_NAMES)).reshape(len(LABEL_NAMES), 1)
FULL_COLOR_MAP = label_to_color_image(FULL_LABEL_MAP)


def run_inference(im: Image, model) -> Tuple[Any, Any]:
    start = time.time()
    resized_im, seg_map = model.run(im)
    end = time.time()
    print(f"Ran deeplab: {(end - start):.3} sec")
    return resized_im, seg_map


def run_visualization(pickle_fpath, model):
    """Inferences DeepLab model and visualizes result."""
    try:
        bgrd, targets = pickle.load(open(pickle_fpath, "rb"))
        bgr = bgrd[0]
        im = Image.fromarray(bgr)
        b, g, r = im.split()
        im = Image.merge("RGB", (r, g, b))
        _ = plt.imshow(im)
        plt.show()
    except pickle.UnpicklingError:
        print(f'UnpicklingError: Please check path: {pickle_fpath}')
        return

    print(f'running deeplab on image {pickle_fpath}')
    resized_im, seg_map = run_inference(im, model)

    vis_segmentation(resized_im, seg_map)


def run_visualization_rgb(rgb_fpath: str, model):
    """Inferences DeepLab model and visualizes result."""
    im = Image.open(rgb_fpath)
    _ = plt.imshow(im)
    plt.show()

    print(f'running deeplab on image {rgb_fpath}')
    resized_im, seg_map = run_inference(im, model)

    vis_segmentation(resized_im, seg_map)



import logging
import shutil
import sys
import time
from pathlib import Path

from azureml.core import Experiment, Workspace
from azureml.core.compute import AmlCompute, ComputeTarget
from azureml.core.compute_target import ComputeTargetException
from azureml.core.run import Run
from azureml.core.script_run_config import ScriptRunConfig

sys.path.append(Path(__file__).parent)

logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)
handler = logging.StreamHandler()
handler.setFormatter(logging.Formatter('%(asctime)s - %(levelname)s - %(message)s - %(pathname)s: line %(lineno)d'))
logger.addHandler(handler)

CWD = Path(__file__).parent
REPO_DIR = Path(__file__).absolute().parents[5]
EVAL_EXPERIMENT_NAME = 'QA-pipeline'
CLUSTER_NAME = 'gpu-cluster'
OVERALL_SAVE_PATH = './outputs'


def copy_dir(src: Path, tgt: Path, glob_pattern: str, should_touch_init: bool = False):
    logger.info("Creating temp folder")
    if tgt.exists():
        shutil.rmtree(tgt)
    tgt.mkdir(parents=True, exist_ok=True)
    if should_touch_init:
        (tgt / '__init__.py').touch(exist_ok=False)

    paths_to_copy = list(src.glob(glob_pattern))
    logger.info(f"Copying to {tgt} the following files: {str(paths_to_copy)}")
    for p in paths_to_copy:
        destpath = tgt / p.relative_to(src)
        destpath.parent.mkdir(parents=True, exist_ok=True)
        shutil.copy(p, destpath)


if __name__ == "__main__":
    # Copy QA src/ dir
    temp_path = CWD / "temp_eval"
    copy_dir(src=CWD / "src", tgt=temp_path, glob_pattern='*.py')

    # Copy common/ folder
    common_dir_path = REPO_DIR / "cgmml/common"
    temp_common_dir = temp_path / "cgmml/common"
    copy_dir(src=common_dir_path, tgt=temp_common_dir, glob_pattern='*/*.py', should_touch_init=True)

    from temp_eval.cgmml.common.model_utils.environment import cgm_environment  # noqa: E402, F401

    workspace = Workspace.from_config()
    run = Run.get_context()

    experiment = Experiment(workspace=workspace, name=EVAL_EXPERIMENT_NAME)

    # Find/create a compute target.
    try:
        # Compute cluster exists. Just connect to it.
        compute_target = ComputeTarget(workspace=workspace, name=CLUSTER_NAME)
        logger.info("Found existing compute target.")
    except ComputeTargetException:
        logger.info("Creating a new compute target...")
        compute_config = AmlCompute.provisioning_configuration(vm_size='Standard_NC6', max_nodes=4)
        compute_target = ComputeTarget.create(workspace, CLUSTER_NAME, compute_config)
        compute_target.wait_for_completion(show_output=True, min_node_count=None, timeout_in_minutes=20)
    logger.info("Compute target: %s", compute_target)

    # parameters used in the evaluation
    script_params = {}
    logger.info("script_params: %s", script_params)
    tags = script_params

    start = time.time()

    cgm_env = cgm_environment(workspace, curated_env_name="cgm-QA-v01", env_exist=True,
                              fpath_env_yml=CWD / "environment.yml")

    script_run_config = ScriptRunConfig(source_directory=temp_path,
                                        compute_target=compute_target,
                                        script='evaluate.py',
                                        arguments=[str(item) for sublist in script_params.items() for item in sublist],
                                        environment=cgm_env)

    # Set compute target.
    script_run_config.run_config.target = compute_target

    # Run the experiment.
    run = experiment.submit(config=script_run_config, tags=tags)

    # Show run.
    logger.info("Run: %s", run)

    # Check the logs of the current run until is complete
    run.wait_for_completion(show_output=True)

    # Print Completed when run is completed
    logger.info("Run status: %s", run.get_status())

    end = time.time()
    logger.info("Total time for evaluation experiment: %d sec", end - start)

    # Download the evaluation results of the model
    GET_CSV_FROM_EXPERIMENT_PATH = '.'
    run.download_files(OVERALL_SAVE_PATH, GET_CSV_FROM_EXPERIMENT_PATH)
    logger.info("Downloaded the result.csv")

    # Delete temp folder
    shutil.rmtree(temp_path)



from setuptools import setup, find_packages

setup(
    name="cgm-ml-common",
    version="3.0.0-rc2",
    author="Markus Hinsche",
    author_email="mhinsche@childgrowthmonitor.com",
    url="https://github.com/Welthungerhilfe/cgm-ml",

    description="ChildGrowthMonitor's ML Common code",
    long_description=open("README.md", "r").read(),
    long_description_content_type="text/markdown",

    classifiers=[
        'Intended Audience :: Healthcare Industry',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Operating System :: OS Independent',
    ],

    packages=find_packages(),

    python_requires=">=3.6",
    setup_requires=['pytest-runner', 'flake8'],
    tests_require=['pytest'],
    include_package_data=True,
)



from azureml.core import Workspace
from azureml.core.model import InferenceConfig, Model
from azureml.core.webservice import AciWebservice  # noqa: E401
from azureml.core.webservice import LocalWebservice
from config_deepensemble_1 import CONFIG

from constants import REPO_DIR

from cgmml.common.model_utils import environment

if __name__ == "__main__":

    workspace = Workspace.from_config()
    model = Model(workspace, name=CONFIG.MODEL_NAME)

    cgm_env = environment.cgm_environment(workspace=workspace, curated_env_name="cgm-env", env_exist=True)

    inference_config_aci = InferenceConfig(
        environment=cgm_env,
        entry_script=str(REPO_DIR / "cgmml/common/endpoints/entry_script_aci.py"),
    )

    if CONFIG.LOCALTEST:
        deployment_config = LocalWebservice.deploy_configuration(port=6789)
    else:
        deployment_config = AciWebservice.deploy_configuration(cpu_cores=1, memory_gb=4)

    service = Model.deploy(workspace, CONFIG.ENDPOINT_NAME, [model],
                           inference_config_aci, deployment_config, overwrite=True,)
    service.wait_for_deployment(show_output=True)
    print(service.swagger_uri)



import pandas as pd
import json
from pathlib import Path
import os
import sys

REPO_DIR = Path(__file__).parents[1].absolute()

# To run the toolkit
# python3 ConvertableData/data_convestion_toolkit.py wfh_girls_p_exp.txt
# wfh_girls_z_exp.txt wfh_girls_2_5_zscores.json Height

# Percentile data file name
percentile_data_path = sys.argv[1]

# SD data file name
sd_data_path = sys.argv[2]

# Output Data filename
output_file_name = sys.argv[3]

# Day if table is wfa,lhfa
# Height if table is wfh
# Lenght if table is wfl
zscore_type = sys.argv[4]


def convert_txt_to_json(percentile_data_path, sd_data_path, output_file_name, zscore_type):
    lsm_sd_data = []

    percentile_data_path = REPO_DIR / 'ConvertableData' / percentile_data_path
    sd_data_path = REPO_DIR / 'ConvertableData' / sd_data_path
    output_file_name = REPO_DIR / 'tables' / output_file_name

    if os.path.exists(output_file_name):
        os.remove(output_file_name)

    df = pd.read_csv(percentile_data_path, sep='\t')
    df2 = pd.read_csv(sd_data_path, sep='\t')
    df = df.drop(['P01', 'P1', 'P3', 'P5', 'P10', 'P15', 'P25', 'P50',
                  'P75', 'P85', 'P90', 'P95', 'P97', 'P99', 'P999'], axis=1)
    df2 = df2.drop([zscore_type, 'SD4neg', 'SD4'], axis=1)
    result = pd.concat([df, df2], axis=1)
    for i in range(len(result)):
        dict = {}
        for j in (result.columns):
            if j == 'Day':
                g = int(result[j][i])
            else:
                g = float("{0:.5f}". format(result[j][i]))
            dict[j] = str(g)
        lsm_sd_data.append(dict)
    with open(output_file_name, 'w') as outfile:
        json.dump(lsm_sd_data, outfile, indent=4)


if __name__ == "__main__":
    convert_txt_to_json(percentile_data_path, sd_data_path,
                        output_file_name, zscore_type)



import os

from bunch import Bunch

CONFIG_NAME = os.path.splitext(os.path.basename(__file__))[0]

# Details of model used for evaluation
MODEL_CONFIG = Bunch(dict(
    EXPERIMENT_NAME='2021q1-depthmap-ensemble-height-95k',
    RUN_IDS=['2021q1-depthmap-ensemble-height-95k_1622230917_290d733e',
             '2021q1-depthmap-ensemble-height-95k_1622230877_02b7cc34',
             '2021q1-depthmap-ensemble-height-95k_1622230838_187c2c8f',
             '2021q1-depthmap-ensemble-height-95k_1622230815_3a6833d4',
             '2021q1-depthmap-ensemble-height-95k_1622230796_d18037bb',
             '2021q1-depthmap-ensemble-height-95k_1622230770_32a7a89c',
             '2021q1-depthmap-ensemble-height-95k_1622230729_c239a326',
             '2021q1-depthmap-ensemble-height-95k_1622230699_886a2a9d',
             '2021q1-depthmap-ensemble-height-95k_1622230664_d9d108b4',
             '2021q1-depthmap-ensemble-height-95k_1622230624_f791db0c',
             '2021q1-depthmap-ensemble-height-95k_1622230551_76ff056b',
             '2021q1-depthmap-ensemble-height-95k_1622230426_12111939',
             '2021q1-depthmap-ensemble-height-95k_1622230397_a2f46051',
             '2021q1-depthmap-ensemble-height-95k_1622230371_7b4cc0a2',
             '2021q1-depthmap-ensemble-height-95k_1622230334_67c64a77'],
    INPUT_LOCATION='outputs',
    NAME='best_model.ckpt',
))


EVAL_CONFIG = Bunch(dict(
    # Name of evaluation
    NAME='2021q1-depthmap-ensemble-height-95k',

    # Experiment in Azure ML which will be used for evaluation

    # Used for Debug the QA pipeline
    DEBUG_RUN=False,

    # Will run eval on specified # of scan instead of full dataset
    DEBUG_NUMBER_OF_SCAN=5,

    SPLIT_SEED=0,
))

# Details of Evaluation Dataset
DATA_CONFIG = Bunch(dict(
    NAME='anon-realtime-testdata',  # Name of evaluation dataset

    IMAGE_TARGET_HEIGHT=240,
    IMAGE_TARGET_WIDTH=180,

    BATCH_SIZE=512,  # Batch size for evaluation
    NORMALIZATION_VALUE=7.5,

    TARGET_INDEXES=[0, 5],  # 0 is height, 1 is weight.
    CODES=['100', '101', '102', '200', '201', '202']
))


# Result configuration for result generation after evaluation is done
RESULT_CONFIG = Bunch(dict(
    # Error margin on various ranges
    # EVALUATION_ACCURACIES = [.2, .4, .8, 1.2, 2., 2.5, 3., 4., 5., 6.]
    ACCURACIES=[.2, .4, .6, 1, 1.2, 2., 2.5, 3., 4., 5., 6.],  # 0.2cm, 0.4cm, 0.6cm, 1cm, ...
    ACCURACY_MAIN_THRESH=1.0,
    COLUMNS=['qrcode', 'artifact', 'scantype', 'GT', 'predicted'],

    USE_UNCERTAINTY=True,  # Flag to enable model uncertainty calculation
    UNCERTAINTY_THRESHOLD_IN_CM=4.,

    # path of csv file in the experiment which final result is stored
    SAVE_PATH=f'./outputs/{CONFIG_NAME}',
))
